#! /usr/bin/perl
#
# COPYRIGHT 2018 Fujitsu Limited  ALL RIGHTS RESERVED
#

use warnings;
use strict;
use File::Temp qw/tempfile/;
use IO::Handle;		# for flush()
use Getopt::Std;

my %opt = (
    D => 0
);
getopts('D', \%opt);

if (open(my $current_mdadm_conf, '<', '/etc/mdadm.conf')) {
    # Note: the file is automatically removed when the program exits
    my ($tmpconf, $tmpconfname) = tempfile;	# /tmp/mdadm.conf

    # grep -v '^DEVICE' /etc/mdadm.conf > /tmp/mdadm.conf
    while (<$current_mdadm_conf>) {
        next if /^DEVICE/;	# Skip DEVICE lines
        print $tmpconf $_;
    }

    # echo DEVICE \$(cd /sys/block; ls -d sd* hd* 2>/dev/null | sed s',\\<,\/dev\/,g') >> /tmp/mdadm.conf"
    if (opendir(my $d, '/sys/block')) {
        my @devices;
        while (my $de = readdir($d)) {
            push @devices, $de if $de =~ /^[sh]d/;
        }
        closedir($d);
        # @devices contains all entries in /sys/block that start with "sd" or "hd"
        if (@devices) {
            # 1) prepend /dev/ in front of each entry in @devices
            # 2) join "DEVICES" and all of these with blanks in between
            print $tmpconf join(' ', 'DEVICES', map { '/dev/' . $_ } @devices), "\n";
        }
    }
    
    $tmpconf->flush();	# flush so mdadm will see all

    # mdadm --examine --scan -c /tmp/mdadm.conf > /tmp/mdadm.tmp
    # 1) execute the mdadm command and return the output ("qx")
    # 2) split the output at any line break
    # 3) collect lines in mdadm_tmp array
    my @mdadm_tmp = split(/[\r\n]+/, qx/mdadm --examine --scan -c $tmpconfname/);
    if ($? != 0) {
        die"$0: \"mdadm --examine --scan -c $tmpconfname\" failed\n";
    }
    close($tmpconf);
    print STDERR "mdadm output:\n", join("\n", @mdadm_tmp, "") if $opt{D};

    # Now that we know everything is OK, build the mdadm.conf proper
    # Rewind current /etc/mdadm.conf
    seek($current_mdadm_conf, 0, 0);

    # One more temporary file. This will hold the new version of /etc/mdadm.conf
    my ($new_mdadm_conf, $fileo) = tempfile();	# /tmp/mdadm.sav

    while (my $line = <$current_mdadm_conf>) {
	print STDERR "mdadm.conf>> $line" if $opt{D};
        if ($line =~ /BEGIN added by LMD/) {
	    print $new_mdadm_conf $line;
            # read and discard next line
            my $dummy = <$current_mdadm_conf>;
            # read next line and ...
            my $line2 = <$current_mdadm_conf>;
            # ... pass to findmatchingline()
	    findmatchingline($line2, \@mdadm_tmp, $new_mdadm_conf) if (defined $line2);
        } else {
            print $new_mdadm_conf $line;
        }
    }
    close($current_mdadm_conf);

    # If we have any output in /tmp/mdadm.sav, then use that file as the new /etc/mdadm.conf

	# The following is probably unneccessary because rewind() will also flush.
    # However, we flush() so "-s" will have no problem.
    $new_mdadm_conf->flush();
    if (-s $fileo > 0) {	# "if size of $fileo is greater than 0" ... */
	print STDERR "setting up new /etc/mdadm.conf:\n" if $opt{D};
        rename('/etc/mdadm.conf', '/etc/mdadm.conf.sav') or die "$0: Failed to rename /etc/mdadm.conf to /etc/mdadm.conf.sav: $!\n";
        # copy /tmp/mdadm.sav to new /etc/mdadm.conf
	seek($new_mdadm_conf, 0, 0);
        if (open(my $dst, '>', '/etc/mdadm.conf')) {
            while (<$new_mdadm_conf>) {
		print STDERR ">>$_" if $opt{D};
                print $dst $_;
            }
            close($dst);
        }
    }

    close($new_mdadm_conf);
}

sub findmatchingline {
    my ($line, $inref, $new_mdadm_conf) = @_;

    if ($line =~ /name=/) {
        my $name = substr ($line, 47, 5);
        foreach (@$inref) {
            if (/$name/) {
                print $new_mdadm_conf $_, "\n";
            }
        }
    }
}
